
//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslc.c
//  MODULE TYPE : Main module
//  STATUS      : In development
//  PURPOSE     : Main module of the ASL compiler
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995  
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.     
//
//            1.00  (Dec 23, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution. 
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Sintax extended according to the NSLC 3.0 specifications.
// 


#include "aslc.h"

// Global objects
AslCompiler compiler;
int flagObject    = FALSE;
int flagExtends   = FALSE;
int flagAssign    = FALSE;
int flagSchema	  = FALSE;
char myArgument[STR_LEN_LINE];
char myStack[STR_LEN_LINE];
char myAssign[STR_LEN_LINE];

//
// Global Symbol Table.
//
/*
char symbolTable[MAX_SYMBOL];
char *symTablePos = symbolTable;
int  symPosition = 0;
*/


//
// External variables and functions
//

extern FILE *yyin, *yyout;		// Yacc input and output files.
extern int yyparse(void);		// Yacc parser.


//
// AslSymbol members.
//

AslSymbol::AslSymbol(void) 
{ 
	/* name = symTablePos;  */
}

void  AslSymbol::setName(char*s) 
{
	name = new char[strlen(s)+1];

	strcpy(name,s);
}

char* AslSymbol::getName(void)
{
	return name; 
}

void AslSymbol::setSymbolType(int t)
{
	symbolType = t;
} 

int  AslSymbol::getSymbolType(void)
{
	return symbolType;
}

int AslSymbol::operator < (AslSymbol s)
{ 
	return (strcmp(name, s.name)<0);  
}

int AslSymbol::operator > (AslSymbol s)
{
	return (strcmp(name, s.name)>0);
}

int AslSymbol::operator == (AslSymbol s)
{
	return (strcmp(name, s.name)==0);
}

void AslSymbol::print(void)
{
	cerr << name << '\n';
}


//
// AslSchemaSymbol members.
//

AslSchemaSymbol::AslSchemaSymbol(void) 
{ 
	setSymbolType(SCHEMA_SYMBOL); 
}

void AslSchemaSymbol::setPrototype(int p) 
{ 
	prototype = p;
}

int AslSchemaSymbol::isPrototype(void)
{ 
	return prototype; 
}


//
// AslStructSymbol members.
//

AslStructSymbol::AslStructSymbol(void) 
{
	setSymbolType(STRUCT_SYMBOL);
}


//
// AslFunctionSymbol members.
//

AslFunctionSymbol::AslFunctionSymbol(void)
{
	setSymbolType(FUNCTION_SYMBOL);
}


//
// AslIdentifierSymbol members.
//

AslIdentifierSymbol::AslIdentifierSymbol(void)
{
	setSymbolType(IDENT_SYMBOL);
}

void AslIdentifierSymbol::setScope(int s)
{
	scope = s;
}

int AslIdentifierSymbol::getScope(void)
{
	return scope;
}

void AslIdentifierSymbol::setType(char* t)
{
	type = new char[strlen(t)];

	strcpy(type,t);
}

char* AslIdentifierSymbol::getType(void)
{
	return type;
}

void AslIdentifierSymbol::print(void)
{
	cerr << getName() << " of type " << type << '\n';
}


//
// Function setLexState is implemented at the end of file aslclex.l
//


//
// AslParser members.
//

AslParser::AslParser(void)
{
	numOfLines    	= 0;
	linePosition  	= 0;
	schemaIndex   	= 0;
	argIndex      	= 0;
	objectIndex	= 0;
	scope	      	= 0;
	arglist		= NULL;
	objectlist	= NULL;
	strcpy(actualExtends,"");
}

int(*AslParser::parse)(void) = (int(*)(void))yyparse;

void AslParser::saveLine(char* l)
{
	int len = strlen(l);

	strcpy(line, l);

	for (int i=0; i<len; i++)
		if(line[i]=='\t') line[i]=' ';
}

char *AslParser::getLine(void)
{
	return line;
}

void AslParser::incNumOfLines(void)
{
	numOfLines++;
}

void AslParser::setNumOfLines(int s)
{
	numOfLines = s;
}

int AslParser::getNumOfLines(void)
{
	return numOfLines;
}

void AslParser::setActualType(char* t)
{
	strcpy(actualType,t);
}

char* AslParser::getActualType(void)
{
	return actualType;
}

void AslParser::setActualIdentifier(char* i)
{
	strcpy(actualIdentifier,i);
}

char* AslParser::getActualIdentifier(void)
{
	return actualIdentifier;
}

void AslParser::setLinePos(int p)
{
	linePosition = p;
}

int AslParser::getLinePos(void)
{
	return linePosition;
}

void AslParser::addLinePos(int p)
{
	linePosition += p;
}

void AslParser::incNumOfSchemas(void)
{
	schemaIndex++;
}

int AslParser::getNumOfSchemas(void)
{
	return schemaIndex;
}

void AslParser::setAccess(int a)
{
	access = a;
}

void AslParser::incScope(void)
{
	 scope ++;
}

void AslParser::decScope(void)
{
	 scope --;
}

int AslParser::getScope(void)
{
	return scope;
}

void AslParser::pushActualSymbol(AslComposedSymbol *a)
{
	actualSymbol[1] = actualSymbol[0];
	actualSymbol[0] = a;
}

AslComposedSymbol *AslParser::popActualSymbol(void)
{
	AslComposedSymbol *temp=actualSymbol[0];

	actualSymbol[0] = actualSymbol[1];

	return temp;
}

void AslParser::insertSchema(char *n, int p)
{
	int res;
	AslSchemaSymbol *s;

	s = new AslSchemaSymbol;
	s->setName(n);
	s->setPrototype(p);

	res = schemaTree.insert(s);

	if (res)
	{
		char buf[STR_LEN_LINE];

		sprintf(buf,"Schema [%s] already defined", s->getName());
		compiler.semanticError(buf);

		delete s;
	}
	else
		pushActualSymbol(s);
}

AslSchemaSymbol* AslParser::searchSchema(char *n)
{
	AslSchemaSymbol *s,*res;	

	s = new AslSchemaSymbol;
	s->setName(n);

	res = schemaTree.search(s);

	delete s;

	return res;
}

void AslParser::printSchemaList(void)
{
	AslComposedSymbol *res;
	AslPtr    *ptr;

	if ((res=schemaTree.getActual()) != NULL)
	{
		ptr = schemaTree.getActualPtr();

		schemaTree.goLeft();
		printSchemaList();

		res->print();

		pushActualSymbol(res);
		actualSymbol[0]->identifierTree.startOnTop();
		//printIdentifierList();

		schemaTree.setActualPtr(ptr);

		schemaTree.goRight();
		printSchemaList();
	}
}

void AslParser::insertIdentifier(char *n)
{
	int res;
	AslIdentifierSymbol *s;

	s = new AslIdentifierSymbol;
	s->setName(n);
	s->setType(actualType);

	setActualIdentifier(n);

	if (!flagExtends)
	  {
		res = actualSymbol[0]->identifierTree.insert(s);
		if (res)
		  {
			char buf[STR_LEN_LINE];
			sprintf(buf,"Identifier [%s] already defined", s->getName());
			compiler.semanticError(buf);
			delete s;
		  }
	  }
}

AslIdentifierSymbol* AslParser::searchIdentifier(char *n)
{
	if (actualSymbol[0]!= NULL)
	{
		AslIdentifierSymbol *s,*res;

		s = new AslIdentifierSymbol;
		s->setName(n);

		res = actualSymbol[0]->identifierTree.search(s);

		delete s;
		
		return res;
	}

	return NULL;
}

void AslParser::printIdentifierList(void)
{
	AslSymbol *res;
	AslPtr    *ptr;

	if ((res=actualSymbol[0]->identifierTree.getActual()) != NULL)
	{
		ptr = actualSymbol[0]->identifierTree.getActualPtr();

		actualSymbol[0]->identifierTree.goLeft();
		printIdentifierList();

		res->print();

		actualSymbol[0]->identifierTree.setActualPtr(ptr);

		actualSymbol[0]->identifierTree.goRight();
		printIdentifierList();
	}
}

void AslParser::insertFunction(char *f)
{
	int res;
	AslSchemaSymbol *temp;
	AslFunctionSymbol *s;

	s = new AslFunctionSymbol;
	s->setName(f);

	temp = (AslSchemaSymbol*)actualSymbol[0];

	res = temp->functionTree.insert(s);

	if (res)
	{
		char buf[STR_LEN_LINE];

		sprintf(buf,"Identifier [%s] already defined", s->getName());
		compiler.semanticError(buf);

		delete s;
	}
	else
		pushActualSymbol(s);
}

AslFunctionSymbol* AslParser::searchFunction(char *n)
{
	AslSchemaSymbol *temp;
	AslFunctionSymbol *s,*res;

	s = new AslFunctionSymbol;
	s->setName(n);

	temp = (AslSchemaSymbol*)actualSymbol[0];

	res = temp->functionTree.search(s);

	delete s;

	return res;
}

void AslParser::processIncludeLines(char *l)
{
	char includeFileName[STR_LEN_LINE];
	int  ans, lineNumber;

	ans = sscanf(l+1,"%d \"%[^\"]\"", &lineNumber ,includeFileName);

	if (ans<2)
	{
		if( strstr(l, "pragma")==NULL )
			compiler.sintaxError("Unknown compiler directive");
	}
	else
	{
		setNumOfLines(lineNumber);
		compiler.setInputFileName(includeFileName);
	}
}

int AslParser::whichTypeIs(char *n)
{
	AslSymbol *res;

	res = searchSchema(n);

	if (res!=NULL)
		return SCHEMA_SYMBOL;

	res = searchIdentifier(n);

	/*if (res!=NULL)
		return IDENT_SYMBOL;*/

	return -1;
}

void  AslParser::setActualSchema(char* s)
{
	strcpy(actualSchema,s);
}

char* AslParser::getActualSchema()
{
	return actualSchema;
}
void  AslParser::setActualAccessOption(char* s)
{
	strcpy(actualAccessOption,s);
}

char* AslParser::getActualAccessOption()
{
	return actualAccessOption;
}

void  AslParser::makeConstructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	getSchemaArguments();				
	compiler.putInOutFile(") ");
	getActualExtends();
	getExtendsArguments();
	if (strcmp(actualExtends,"") != 0) compiler.putInOutFile(")");
	getObjects();						
	compiler.putInOutFile(" \n{\n");
	compiler.putInOutFile(myAssign);
	compiler.putInOutFile("\n}\n\n");
}

void  AslParser::makeDestructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(")\n{\n");
	compiler.putInOutFile("\n}\n\n");
}


void  AslParser::makeConstructorDecl(char* s)
{
	compiler.putInOutFile("\n\t");
	compiler.putInOutFile("public: ");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	getSchemaArgumentsDecl();
	compiler.putInOutFile(");\n");
}

void  AslParser::makeDestructorDecl(char* s)
{
	compiler.putInOutFile("\tpublic: ~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(");\n\n");
}


AslArgument* AslParser::insertArg(char* type, char* access, char* ident, int flagArgument)
{
	AslArgument *tmp;
	tmp = new AslArgument;
	tmp->setArgument(type,access,ident,flagArgument);
	if (arglist==NULL)
	  {
		arglist=tmp;
	  }
	else
	  {
		mylast->next=tmp;
	  }
	mylast=tmp;
	argIndex++;
	return arglist;
}

AslArgument* AslParser::removeArg(void)
{
	AslArgument *tmp;
	tmp=arglist;
	arglist=arglist->next;
	delete tmp;
	argIndex--;
	return arglist;
	
}

void AslParser::getSchemaArguments(void)
{
	AslArgument *root;

	root=arglist;
	while ((root!=NULL) && (root->getArgFlag() == FALSE))
	  {
		compiler.putInOutFile(root->getType());
		compiler.putInOutFile(root->getAccess());
		compiler.putInOutFile(" ");
		compiler.putInOutFile(root->getIdent());
		root=removeArg();
	 	if ((root->next != NULL) && (root->getArgFlag() == FALSE)) compiler.putInOutFile(", ");
	  }
}

void AslParser::getSchemaArgumentsDecl(void)
{
	AslArgument *root;

	root=arglist;
	while ((root!=NULL) && (root->getArgFlag() == FALSE))
	  {
		compiler.putInOutFile(root->getType());
		compiler.putInOutFile(root->getAccess());
		/*compiler.putInOutFile(" ");
		compiler.putInOutFile(root->getIdent());*/
		root = root->next;
	 	if ((root != NULL) && (root->getArgFlag() == FALSE)) compiler.putInOutFile(", ");
	  }
}

void AslParser::getExtendsArguments(void)
{
	AslArgument *root;

	root=arglist;
	while ((root!=NULL) && (root->getArgFlag() == TRUE))
	  {
		compiler.putInOutFile(root->getIdent());
		root=removeArg();
		if ((root != NULL) && (root->getArgFlag() == TRUE)) compiler.putInOutFile(",");
	  }
}
	
AslObject* AslParser::insertObject(char* typeobj,char* accessobj,char* nameobj)
{
	AslObject *tmp;
	tmp = new AslObject;
	tmp->setParam(typeobj,accessobj,nameobj);
	if (objectlist==NULL)
	  {
		objectlist=tmp;
	  }
	else
	  {
		objlast->nextObject=tmp;
	  }
	objlast=tmp;
	objectIndex++;
	return objectlist;

}

void AslParser::insertObjectArguments(char* argument)
{
	objlast->setObjectArguments(argument);
}

AslObject* AslParser::removeObject(void)
{
	AslObject *tmp;
	tmp=objectlist;
	objectlist=objectlist->nextObject;
	delete tmp;
	objectIndex--;
	return objectlist;
}

void AslParser::getObjects(void)
{
	AslObject *root;
	root=objectlist;
	while (root!=NULL)
	  {
		compiler.putInOutFile(", ");
		compiler.putInOutFile(root->getObjectName());
		compiler.putInOutFile("(");
		compiler.putInOutFile(root->getObjectArguments());
		compiler.putInOutFile(")");
		root=removeObject();
	  }

}

AslObject* AslParser::insertObjectParam(char* typeparam,char* accessparam,char* param)	
{
	compiler.putInOutFile(typeparam);
	compiler.putInOutFile("accessparam");
	compiler.putInOutFile("param");
	compiler.putInOutFile(" ");
}

void AslParser::setActualExtends(char* inherit)
{
	strcpy(actualExtends,inherit);
}

void AslParser::setActualExtendsArgs(char* myargs)
{
	strcat(actualExtends,myargs);
}

void AslParser::getActualExtends(void)
{
	char mio[STR_LEN_LINE];

	if (strcmp(actualExtends,"") == 0) 
	  {
		compiler.putInOutFile(actualExtends);		
	  }
	else
	  {
		strcpy(mio,": ");
		strcat(mio,actualExtends);
		compiler.putInOutFile(mio);
		compiler.putInOutFile("(");
	  }
}





//
// AslArgument methods
//

AslArgument::AslArgument(void)
{
	strcpy(type,"");
	strcpy(access,"");
	strcpy(ident,"");
	argFlag = 0;
	next 	= NULL;
}

AslArgument::~AslArgument(void)
{
}

void AslArgument::setArgument(char* type1,char* access1,char* ident1, int flagi)
{
	strcpy(type,type1);
	strcpy(access,access1);
	strcpy(ident,ident1);
	argFlag = flagi;
}

char* AslArgument::getType(void)
{
	return type;
}

char* AslArgument::getAccess(void)
{
	return access;
}

char* AslArgument::getIdent(void)
{
	return ident;
}

int AslArgument::getArgFlag(void)
{
	return argFlag;
}



//
// AslObject methods
//

AslObject::AslObject(void)
{
	nextObject = NULL;
}

AslObject::~AslObject(void)
{
}

void AslObject::setParam(char* typeobject,char* accessobject,char* object)
{
	strcpy(type,typeobject);
	strcpy(access,accessobject);
	strcpy(name,object);

}

char* AslObject::getObjectType(void)
{
	return type;
}

char* AslObject::getObjectAccess(void)
{
	return access;
}

char* AslObject::getObjectName(void)
{
	return name;
}


void AslObject::setObjectArguments(char* arg)
{
	strcpy(objectArguments,arg);
}

char* AslObject::getObjectArguments(void)
{
	return objectArguments;
}







//
// AslCompiler methods
//

AslCompiler::AslCompiler(void)
{
	numOfDefines  = 0;
	numOfIncludes = 0;
	numOfFiles    = 0;
	numOfErrors   = 0;

	verbose       = FALSE;
	logging       = FALSE;
}

void AslCompiler::copyRight(void)
{
	cout<<"\nASL to C++ Compiler, V1.0\n"
      	    <<"(c) Copyright 1998, Instituto Tecnologico Autonomo de Mexico\n\n";
}

void AslCompiler::help(void)
{
	copyRight();
	cout<<"Usage: "
	    <<"aslc [-v]"
	    <<"[-I path][-D definition][-e logName][-o outName] "
	    <<"fileNames\n\n";
}

void AslCompiler::parseArguments(int argc, char *argv[])
{
	int option, ans=TRUE, out=FALSE;
	GetOpt getopt(argc, argv, "D:e:I:o:v");

	while ((option = getopt()) != EOF)
	{
		switch (option)
		{
			case 'D': if (numOfDefines < MAX_OF_DEFINES)
				  	strcpy(defines[numOfDefines++], getopt.optarg);
				  else
					operativeError("Too many defines in command line","AslCompiler::parseArguments");
				  break;

			case 'I': if (numOfIncludes < MAX_OF_DEFINES)
					strcpy(includes[numOfIncludes++], getopt.optarg);
				  else
					operativeError("Too many includes in command line","AslCompiler::parseArguments");
				  break;

			case 'e': if (!logging)
				  {
				  	logging = TRUE;
				  	strcpy(logName, getopt.optarg);

					logFile = fopen(logName, "w");
					if (logFile == NULL)
	    			  	{
						sprintf(buf,"Can't open log file %s \n",logName);
						operativeError(buf,"AslCompiler::parseArguments");
				  	}
	    			  	fclose(logFile);

				  	freopen(logName, "w+", stderr);
				  }
				  break;

			case 'o': if (!out)
				  {
					out = TRUE;
				 	setOutputFileName(getopt.optarg);

					fclose(yyout);

				  	yyout = fopen(outputFileName, "w");
				  	if (yyout == NULL)
				  	{
						sprintf(buf,"Can't open file %s for ouput\n",outputFileName);
						operativeError(buf,"AslCompiler::parseArguments");
				  	}
				  }

				  break;

			case 'v': if (!verbose)
				  {
					copyRight();
				  	verbose = TRUE;
				  }

				  break;

			default:  ans = FALSE;
				  break;
      		}
	}

	numOfFiles = argc - getopt.optind;

	if (argc==1 || ans == FALSE)
	{
		help();
		exit(1);
	}

	if (numOfFiles == 0)
		operativeError("No input files \n","AslCompiler::parseArguments");

	fileNames = &argv[getopt.optind];
}

void AslCompiler::preCompile(void)
{
	char temp[STR_LEN_LINE], tempFileName[STR_LEN_LINE];
	int res, i;

	sprintf(buf,"gcc -E ");

	for(i=0; i < numOfIncludes; i++)
	{
		sprintf(temp,"-I%s ",includes[i]);
		strcat(buf, temp);
	}

	for(i=0; i < numOfDefines; i++)
	{
		sprintf(temp,"-D%s ",defines[i]);
		strcat(buf, temp);
	}

	sprintf(tempFileName,"%s_%d_temp.c",inputFileName, getpid());

	sprintf(temp, "-x c++ %s -o %s", inputFileName, tempFileName);
	strcat(buf, temp);

	if (verbose)
		cout << "Precompiling file " << getInputFileName() << "\nWith command: " << buf << "\n";

	res = system(buf);
	
	if (res==256)
		operativeError("Can't execute precompiler without errors","AslCompiler::preCompile");

	fclose(yyin);

	yyin  = fopen(tempFileName,"r");
	if (yyin==NULL)
		operativeError("Can't open temporal input file", "AslCompiler::preCompile");
}

int AslCompiler::isVerboseSet(void)
{
	return verbose;
}

int AslCompiler::getNumOfFiles(void)
{
	return numOfFiles;
}

char *AslCompiler::getFileName(int i)
{
	return fileNames[i];
}

void AslCompiler::setInputFileName(char* f)
{
	strcpy(inputFileName, f);
}

char *AslCompiler::getInputFileName(void)
{
	return inputFileName;
}

void AslCompiler::setOutputFileName(char* f)
{
	strcpy(outputFileName, f);
}

char *AslCompiler::getOutputFileName(void)
{
	return outputFileName;
}

void AslCompiler::sintaxError(char* s)
{
	int i;

	cerr << "\nERROR (sintax) in file " << compiler.getInputFileName()
	     <<	" near line " << compiler.parser.getNumOfLines() << "\n\n"
	     << "Description: " << s << "\n       Line: " << compiler.parser.getLine() 
	     << "\n      Error: " ;

	for (i=0; i < compiler.parser.getLinePos(); i++)
		cerr << '-' ;

	cerr << "^\n" ;

	numOfErrors++;

	if (numOfErrors>MAX_NUM_OF_ERRORS)
	{	
		cerr << "\nFATAL ERROR\n\n"
		     << "Description: " << "Too many errors to continue compiling \n"
		     << "\nTerminating compilation with errors \n\n";

		exit(1);
	}
}

void AslCompiler::semanticError(char* s)
{
	int i;

	cerr << "\nERROR (semantic) in file " << compiler.getInputFileName()
	     <<	" near line " << compiler.parser.getNumOfLines() << "\n\n"
	     << "Description: " << s << "\n       Line: " << compiler.parser.getLine() 
	     << "\n      Error: " ;

	for (i=0; i < compiler.parser.getLinePos(); i++)
		cerr << '-' ;

	cerr << "^\n" ;

	numOfErrors++;

	if (numOfErrors>MAX_NUM_OF_ERRORS)
	{	
		cerr << "\nFATAL ERROR\n\n"
		     << "Description: " << "Too many errors to continue compiling \n"
		     << "\nTerminating compilation with errors \n\n";

		exit(1);
	}
}

void AslCompiler::operativeError(char* s, char* f)
{
	int i;

	cerr << "\nERROR (operative) in function " << f << "\n\n"
	     << "Description: " << s
	     << "\n      Error: " ;

	perror("");
	cerr << "\nTerminating compilation with errors \n\n";

	exit(1);
}

void AslCompiler::putInOutFile(char *s)
{
	fprintf(yyout,"%s",s);
}

void AslCompiler::deleteTempFiles(void)
{
	char tempFileName[STR_LEN_LINE];

	for (int i=0; i < numOfFiles; i++)
	{
		sprintf(tempFileName, "%s_%d_temp.c", fileNames[i], getpid());
		unlink(tempFileName);
	}
}

int AslCompiler::getNumOfErrors(void)
{
	return numOfErrors;
}

void AslCompiler::printCompilerStatistics(void)
{
	cerr << "\nSchemas " << compiler.parser.getNumOfSchemas() << ":\n\n";

	parser.schemaTree.startOnTop();
	parser.printSchemaList();
}


//
// Main compiler function
//

void main(int argc, char *argv[])
{
	int files;

	compiler.parseArguments(argc, argv);

	files = compiler.getNumOfFiles();

	for (int i=0; i < files; i++)
	{
		compiler.setInputFileName(compiler.getFileName(i));
		compiler.preCompile();

		if (compiler.isVerboseSet())
			cout << "Transforming file " << compiler.getInputFileName() <<" in ASL to C++ \n";

		compiler.parser.parse();
	}

	compiler.deleteTempFiles();

	//compiler.printCompilerStatistics();
	if (compiler.getNumOfErrors()>0)
		exit(1);
}


