
//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslc.h
//  MODULE TYPE : Header of main module
//  STATUS      : In development
//  PURPOSE     : Main module definitions
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995  
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution. 
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Sintax extended according to the NSLC 3.0 specifications.
// 


#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <GetOpt.h>

#include "aslbtree.h"
#include "aslstack.h"

const int STR_LEN_LINE      = 256;	// Maximum line len.
const int MAX_OF_INCLUDES   = 10;	// Maximum number of include paths.
const int MAX_OF_DEFINES    = 10;	// Maximum number of defines.
const int MAX_NUM_OF_ERRORS = 100;	// Maximum number of permited errors.	


//
// Boolean constants.
//

const int TRUE  = 1;
const int FALSE = 0;


//
// Schema access constans (external, internal).
//

const int EXT = 0;
const int INT = 1;


//
// Symbol type constants.
//

const int SCHEMA_SYMBOL     = 0;
const int STRUCT_SYMBOL     = 1;
const int FUNCTION_SYMBOL   = 2;
const int IDENT_SYMBOL      = 3;


//
// Classes prototype.
//

class AslCompiler;
class AslSymbol;
class AslSchemaSymbol;
class AslIdentifierSymbol;
class AslFunctionSymbol;
class AslStructSymbol;


//
// Classes for symbol management.
//

class AslSymbol
{
	char *name;					// Symbol name.
	int  symbolType;				// Symbol type: Schema
 
public:
	AslSymbol(void);

	void  setName(char*s);				// Set symbol name.
	char* getName(void);				// Get symbol name.

	void setSymbolType(int t);			// Set symbol type.
	int  getSymbolType(void);			// Get symbol type.

	virtual int operator < (AslSymbol);		// Overloaded operators 
	virtual int operator > (AslSymbol);		// for searching in the
	virtual int operator == (AslSymbol);		// symbol tree.

	virtual void print(void);			// Print symbol information.
};

class AslIdentifierSymbol : public AslSymbol
{
	char* type;					// Identifier type
	int  scope;					// Identifier scope

public:
	AslIdentifierSymbol(void);

	void setScope(int);				// Set identifier scope.
	int  getScope(void);				// Get identifier scope.

	void  setType(char*);				// Set identifier type.
	char* getType(void);				// Get identifier type.

	void print(void);				// Print identifier information.
};

class AslComposedSymbol : public AslSymbol
{
public:
	AslBTree<AslIdentifierSymbol> identifierTree;	// Identifier symbol tree.
};

class AslStructSymbol : public AslComposedSymbol
{
public:
	AslStructSymbol(void);

	AslBTree<AslIdentifierSymbol> identifierTree;	// Identifier symbol tree.
};

class AslFunctionSymbol : public AslComposedSymbol
{
public:
	AslFunctionSymbol(void);
};

class AslSchemaSymbol : public AslComposedSymbol
{
	int  prototype;					// Determine if the declaration
							// is a schema prototye or a
							// complete declaration.
public:
	AslSchemaSymbol(void);

	AslBTree<AslStructSymbol>     structTree;	// Struct symbol tree.
	AslBTree<AslFunctionSymbol>   functionTree;	// Function symbol tree.

	void setPrototype(int p);			// Set if it is prototye or not.
	int  isPrototype(void);				// Determine if it is prototype or not.
};


//
// Classes used by the parser
//

class AslArgument					// AslArgument hold a list of arguments for every Schema
{
public:
	int argFlag;					// FALSE if SchemaArgument, TRUE if ExtendsArgument
	char type[STR_LEN_LINE];			
	char access[STR_LEN_LINE];
	char ident[STR_LEN_LINE];
	AslArgument *next;

public:
	AslArgument(void);
	~AslArgument(void);
	void setArgument(char*,char*,char*,int);
	char *getType(void);
	char *getAccess(void);
	char *getIdent(void);
	int  getArgFlag(void);
};

class AslObject						// AslObject hold a list of objects for every Schema
{
public:
	char type[STR_LEN_LINE];
	char access[STR_LEN_LINE];
	char name[STR_LEN_LINE];
	char objectArguments[STR_LEN_LINE]; 
	AslObject *nextObject;

public:
	AslObject(void);
	~AslObject(void);
	void setParam(char*,char*,char*);
	void setObjectArguments(char*);
	char *getObjectType(void);
	char *getObjectAccess(void);
	char *getObjectName(void);
	char *getObjectArguments(void);
};

class AslParser
{
	int numOfLines;       				// Line counter (for error flagging).
	int linePosition;     				// Position in the actual line (for error flagging).

	int access;	     				// Determine if analized members are external or internal.
	int scope;	      				// Determine the scope of the analized member.

	int schemaIndex;      				// Schema counter.
	int argIndex;					// Arguments counter
	int objectIndex;				// Objects counter
	
	AslComposedSymbol* actualSymbol[2];		// Parent symbol.		
	
	

	char line[STR_LEN_LINE];			// Line buffer (for error flagging).
	char actualAccessOption[STR_LEN_LINE];		// Actual accessOption
	char actualExtends[STR_LEN_LINE];		// Actual inherit
	char actualIdentifier[STR_LEN_LINE];		// Actual identifier
	char actualSchema[STR_LEN_LINE];		// Actual schema_name
	char actualType[STR_LEN_LINE];			// Actual type_name

public:
	AslArgument *arglist,*mylast;			// Pointers to Argumentlist 
	AslObject *objectlist,*objlast;			// Pointers to Objectlist

	AslParser(void);

	static int (*parse)(void); 			// Yacc parser.

	void  saveLine(char*);				// Save actual line being analized.
	char *getLine(void);				// Get actual line being analized.

	void  incNumOfLines(void);			// Increment line counter.
	void  setNumOfLines(int);			// Set line counter.
	int   getNumOfLines(void);			// Get line counter.

	void  setActualType(char*);			// Set actual type being analized.
	char *getActualType(void);			// Get actual type being analized.

	void  setActualIdentifier(char*);		// Save last identifier being analized.
	char *getActualIdentifier(void);		// Get last indetifier being analized.

	void  setLinePos(int);				// Set actual line position.
	int   getLinePos(void);				// Get actual line position.
	void  addLinePos(int);				// Move actual line position.

	void  setLexState(int);				// Lexical analizer state setting.

	void  incNumOfSchemas(void);			// Increment schema counter.
	int   getNumOfSchemas(void);			// Get schema counter.

	void  setAccess(int);				// Set actual schema access.

	void  incScope(void);				// Increment member scope.
	void  decScope(void);				// Decrement member scope.
	int   getScope(void);				// Get actual scope.

	void  pushActualSymbol(AslComposedSymbol*); 	// Save actual analized symbol.
	AslComposedSymbol* popActualSymbol(void);	// Get actual analized symbol.

	AslBTree<AslSchemaSymbol> schemaTree;   	// Schema symbol tree.

	void  insertSchema(char*, int);			// Insert schema symbol in the tree.
	AslSchemaSymbol *searchSchema(char*);		// Search schema symbol in the tree.
	void  printSchemaList(void);			// Print schema tree (debugging).
	
	void  insertIdentifier(char*);			// Insert identifier symbol in the tree.
	AslIdentifierSymbol *searchIdentifier(char*);	// Search identifier symbol in the tree.
	void  printIdentifierList(void);		// Print identifier tree (debugging).

	void  insertFunction(char*);			// Insert identifier symbol in the tree.
	AslFunctionSymbol *searchFunction(char*);	// Search identifier symbol in the tree.

	void processIncludeLines(char*);		// Process include lines to find which file
							// we are compiling.

	int   whichTypeIs(char*);			// Determine which type the identifier is.

	void  setActualSchema(char*);			// Set schema symbol
	char *getActualSchema(void);			// Get schema symbol

	void  setActualAccessOption(char*);		// Set accessOption
	char *getActualAccessOption();			// Get accessOption

	void  makeConstructorDecl(char*);		// Make constructor declaration
	void  makeDestructorDecl(char*);		// Make destructor declaration
	void  makeConstructor(char*);			// Make constructor
	void  makeDestructor(char*);			// Make destructor	

	AslArgument* insertArg(char*,char*,char*,int);	// Insert an argument in the argumentlist
	AslArgument* removeArg(void);			// Remove the first argument of the argumentlist
	void getSchemaArguments(void);			// Get all the Schema arguments and put them in Constructor
	void getSchemaArgumentsDecl(void);		// Get all the Schema arguments and put them in Constructor decl
	void getExtendsArguments(void);			// Get all the Extends arguments and put them in Constructor

	AslObject* insertObject(char*,char*,char*);	// Insert an object in the list of the Schema
	AslObject* insertObjectParam(char*,char*,char*); //Insert the parameters of the object
	AslObject* removeObject(void);			// Remove the object from the list
	void insertObjectArguments(char*);		// Insert the arguments of the object
	void getObjects(void);				// print the list of objects

	void setActualExtends(char*);			// Set inherit definition	
	void setActualExtendsArgs(char*);		// Set inherit argumentList
	void getActualExtends(void);			// Get inherit definition	

};


//
//  Class for compiler management
//

class AslCompiler
{
	int numOfFiles;					// Number of input files.
	int numOfDefines;				// Number of defines.
	int numOfIncludes;				// Number of includes.

	char logName[STR_LEN_LINE];			// Log file name.
	char defines[MAX_OF_INCLUDES][STR_LEN_LINE];	// Includes vector.
	char includes[MAX_OF_DEFINES][STR_LEN_LINE];	// Defines vector.

	char **fileNames;				// Asl input files.

	FILE *logFile;					// File that would contain standard error messages.

	int logging;					// If true redirect standar error to logName.
	int verbose;					// If true write information about compiling process.

	int numOfErrors;				// Num of current errors.

	char inputFileName[STR_LEN_LINE];
	char outputFileName[STR_LEN_LINE];		// Output c++ file name.

public:	
	AslCompiler(void);

	AslParser parser;				// Asl parser and it's functions. 

	char buf[STR_LEN_LINE];				// General propose buffer.

	void  copyRight(void);
	void  help(void);

	void  parseArguments(int,char**);		// Parse line argument.
	void  preCompile(void);				// Input file precompiling.

	int   isVerboseSet(void);			// Know if verbose flag is set.

	int   getNumOfFiles(void);			// Get number of input files.
	char* getFileName(int);				// Get file name with index i.

	int   getNumOfErrors(void);			// Get number of compile errors.

	void  setInputFileName(char*);			// Set input file name.
	char* getInputFileName(void);			// Get input file name.
	void  setOutputFileName(char*);			// Set output file name.
	char* getOutputFileName(void);			// Set output file name.

	void  sintaxError(char *);			// Sintax error flagging.
	void  semanticError(char *);			// Semantic error flagging.
	void  operativeError(char *, char *);		// Operative error flagging.

	void  deleteTempFiles();			// Remove temporal files.

	void  putInOutFile(char*);			// Write in the c++ output file.
	void  printCompilerStatistics(void);		// Print compiler statistics.

	/*void  makeConstructor(char*);			// Make Constructor
	void  makeDestructor(char*);			// Make Destructor*/

};


//
// Global variables.
//

extern AslCompiler compiler;
extern int flagObject;
extern int flagExtends;
extern int flagAssign;
extern int flagSchema;
extern char myArgument[STR_LEN_LINE];
extern char myStack[STR_LEN_LINE];
extern char myAssign[STR_LEN_LINE];
										


