/* SCCS  %W%---%E%--%U% */
//
//	nsl_block_stim_3d.c
//

# include "nsl_all_kernel_include.h"
# include "nsl_all_input_include.h"

nsl_block_stim_3d::nsl_block_stim_3d(nsl_input_3d* l) 
	: nsl_stim_3d(l)
{
	set_type("block_stim_3d");

	itransx = 0;
	itransy = 0;
	iangle = 0;
	angle = 0;
}
nsl_block_stim_3d::~nsl_block_stim_3d()
{
}

int nsl_block_stim_3d::proc()
{
 	int i,j,k;
	float x,y,kx0,kx1,ky0,ky1;
	int qi0,qj0,qic,qjc,qk0,qkc;
	double pi = 3.1415926535;

	nsl_input_3d* Q;
	NslFloat2* Q2;

	if (m_parent != NULL) {
		if ((Q = (nsl_input_3d*) m_parent->get_m_parent()) != NULL) {
			Q2 = Q->get_xzview_ptr();
		}
		else
			cmd_error("nsl_block_stim_3d: NULL 3d layer parent");
	}
	else
		cmd_error("nsl_block_stim_3d: NULL parent");
	
	if (Q == NULL || Q2 == NULL)
	{
		cmd_error("nsl_block_stim_3d: NULL parent layer");
		return 1;
	}

	int xz = Q->get_xz(); // zero x element
	int yz = Q->get_yz(); // zero y element
	int zz = Q->get_zz();
	
	float mdx = Q->get_dx(); // x interval 
	float mdy = Q->get_dy(); // y interval
	float mdz = Q->get_dz(); // z interval

	int idx = (int)(dx.get_value()/mdx+0.5); // int width
	int idy = (int)(dy.get_value()/mdy+0.5); // int depth
	int idz = (int)(dz.get_value()/mdz+0.5); // int height

	idx = (idx >= 1) ? idx-1 : 0; // >= 0
	idy = (idy >= 1) ? idy-1 : 0; // >= 0
	idz = (idz >= 1) ? idz-1 : 0; // >= 0

	int qimax = Q->get_imax();
	int qjmax = Q->get_jmax();

	if (spec_type == NSL_CENTER) // reference xc,yc
	{
		qjc = (int)(txc.get_value()/mdx+0.5) + xz;  // int left corner
		qic = (int)(tyc.get_value()/mdy+0.5) + yz;  // int top corner
		qkc = (int)(tzc.get_value()/mdz+0.5) + zz;  

		qj0 = qjc - idx/2;
		qi0 = qic - idy/2;
		qk0 = qkc - idz/2;
	}
	else // reference x0,y0
	{
		qj0 = (int)(tx0.get_value()/mdx+0.5) + xz;  // int left corner
		qi0 = (int)(ty0.get_value()/mdy+0.5) + yz;  // int top corner
		qk0 = (int)(tz0.get_value()/mdz+0.5) + zz; 

		qjc = qj0 + idx/2;
		qic = qi0 + idy/2;
		qkc = qk0 + idz/2;
	}

	int qj1 = qj0 + idx;
	int qi1 = qi0 + idy;
	int qk1 = qk0 + idz;

	float w = (Q->get_input_layer()->get_overlapfg() == 1) ? 1.0 : 0.0;

    for (i = qi0; i <= qi1; i++) {
        for (j = qj0; j <= qj1; j++) {
            if (0 <= i && i < qimax && 0 <= j && j < qjmax)
                Q->elem(i,j) = w*Q->elem(i,j) + val;
	    }
	}

	if (spec_view == NSL_ON) {
/*
		itransx = qjc;
		itransy = qic;
		iangle = angle.get_value();
		iangle = iangle*pi/180; // switch to radians

		Q->set_transx(itransx);
		Q->set_transy(itransy);
		Q->set_angle(iangle);
*/
		return 1; // if view stim do not add to xz view
	}

	itransx = Q->get_transx();
	itransy = Q->get_transy();
	angle = Q->get_angle();
	iangle = angle*pi/180; // switch to radians

	if (itransy > qic) {
		transform(ky0,kx0,(float)qic,(float)qjc); // rotate according to iangle
		fill(Q2,nsl_rint(ky0),nsl_rint(ky0),nsl_rint(kx0),nsl_rint(kx0),qk0,qk1); // single element fill 
	}

	return 1;
}

void nsl_block_stim_3d::fill(NslFloat2* Q2,int y0,int y1,int x0,int x1,int qk0,int qk1)
{
	int x,y; // x0,x1,y0,y1;
	int k;
	double pi = 3.1415926535;

	// make field 45 deg 
	float alfa = pi/4;
	// number of degrees in one half of xview
	int deg = 40;

	float nbeta = iangle - alfa; // angle from ortho
	float pbeta = iangle + alfa; // angle from ortho
	float m0 = tan(iangle);
	float mn = tan(nbeta);
	float mp = tan(pbeta);

	if (mn == 0 || mp == 0)
		return;

	int kimax = Q2->get_imax();
	int kjmax = Q2->get_jmax();

	int k2jmax = kjmax/2;

	for (x = x0; x <= x1; x++) {
		for (y = y0; y <= y1;  y++) {
/* The following section limits the visual field
			if (y == 0)
				continue;
			float m = x/y; // tan of new point (original x/y)
			if (m < mn || m > mp) // beyond visual field
				continue;
*/
			// perspective projection
//			int xd = nsl_rint(x*deg/y/mn);
			// parallel projection
			int xd = x;

			if (-k2jmax < xd && xd < k2jmax) 
				for (k = qk0; k <= qk1; k++)
					if (0 <= k && k < kimax)
						Q2->elem(kimax-k-1,k2jmax+xd) = val; // xzview
		}
	}
}

void nsl_block_stim_3d::transform(float& yt,float& xt,float i,float j)
{
	double cosa = cos(iangle);
	double sina = sin(iangle);
	// rotate by -iangle
	xt = (j-itransx)*cosa - (i-itransy)*sina; // + itransy;
	yt = (i-itransy)*cosa + (j-itransx)*sina; // + itransx;
}

int nsl_block_stim_3d::print(ostream& out) const
{
	return print_block_stim_3d(out);
}
int nsl_block_stim_3d::print_block_stim_3d(ostream& out) const
{
 	out << "set stim val " << val << "\n";
	out << "set stim dx " << dx.get_value() << "\n";
	out << "set stim dy " << dy.get_value() << "\n";
	out.flush();

	print_stim_3d(out);

	return 1;
}
int nsl_block_stim_3d::print_status(ostream& out) const
{
	return print_block_stim_3d_status(out);
}
int nsl_block_stim_3d::print_block_stim_3d_status(ostream& out) const
{
	return print_stim_3d_status(out);
}
 
