/* SCCS  @(#)nsl_input_vector.C	1.1---95/08/13--20:55:06 */
//
//	nsl_input_vector.c
//

# include "nsl_all_kernel_include.h"
# include "nsl_all_input_include.h"

nsl_input_vector::nsl_input_vector(const char* str,nsl_public* parent,const int i,
	row_col_enum vtype,int sfg,int fg,int rfg) : 
	nsl_num_vector(str,parent,i,vtype,sfg,fg,0), // rfg
	xz("xz",this),	// *** should be INT ***
	dx("dx",this)
{
	set_type("input_vector");

	layer_type = 1; // input layer
     	xz = 0;
	dx = 1.0;

	input_layer = new nsl_input_layer("input_layer",this);
	if (m_children_list == NULL)
		m_children_list = new nsl_base_list(1);
	m_children_list->append(input_layer);
}	
nsl_input_vector::~nsl_input_vector()
{
}

// input vector assignment (input_vector = vector is NOT derived !!)

NslFloat1& nsl_input_vector::operator=(const nsl_input_vector& a)
{
	if (a.get_imax() != imax)
	{
		cmd_error("Error in nsl_input_vector::operator=");
 		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return *this;
	}

	for (int i = 0; i < imax; i++)
		v[i] = a[i];

	return *this;
}
NslFloat1& nsl_input_vector::operator=(const NslFloat1& a)
{
	if (a.get_imax() != imax)
	{
		cmd_error("Error in nsl_input_vector::operator=");
 		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return *this;
	}

	for (int i = 0; i < imax; i++)
		v[i] = a[i];

	return *this;
}
NslFloat1& nsl_input_vector::operator=(const NslFloat0& a)
{
	int imx = get_imax();  

	num_type val = a.get_value();

	for (int i = 0; i < imx; i++)
		v[i] = val;

	return *this;
}
NslFloat1& nsl_input_vector::operator=(const num_type a)
{
	int imx = get_imax();

	for (int i = 0; i < imx; i++)
		v[i] = a;

	return *this;
}
NslFloat1& nsl_input_vector::operator=(const int a)
{
	int imx = get_imax();

	for (int i = 0; i < imx; i++)
		v[i] = (num_type) a;

	return *this;
}
 
int nsl_input_vector::run()
{
	if (input_layer != NULL)
		input_layer->run();
	else
		cmd_error("NULL input stim");

	return 1;
}

// print status

int nsl_input_vector::print_status(ostream& out) const
{
	return print_input_vector_status(out);
}
int nsl_input_vector::print_input_vector_status(ostream& out) const
{
	nsl_vector::print_status(out);

	if (input_layer != NULL)
		input_layer->print_status(out);
	else
		cmd_error("NULL input stim");

	return 1;
}
int nsl_input_vector::print(ostream& out) const
{
	nsl_vector::print(out);

	out << "set input_layer xz " << xz.get_value() << "\n";
	out << "set input_layer dx " << dx.get_value() << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out);
	else
		cmd_error("NULL input stim");

  	return 1;
}
int nsl_input_vector::print(ostream& out,int i) const
{
	nsl_vector::print(out);

	out << "set input_layer xz " << xz.get_value() << "\n";
	out << "set input_layer dx " << dx.get_value() << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out,i);
	else
		cmd_error("NULL input stim");

  	return 1;
}
