/* SCCS  @(#)nsl_matrix_div_library.C	1.1---95/08/13--20:48:15 */
//
//	nsl_matrix_div_library.C
//
/*
# include "nsl_all_kernel_include.h"

// matrix pointwise division

nsl_matrix operator/(const nsl_matrix& a,const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator/\n";
        cout << "// (matrix) " << a.get_name() << " / (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("*** nsl_matrix::operator/ ***");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
	   {
		if (b[i][j] == 0)
		{
			cmd_error("*** nsl_matrix::operator/ ***");
			b.print_status(cerr);
			b.print(cerr);
			cmd_error("Division by zero");
			return c;
		}
		c[i][j] = a[i][j] / b[i][j];
	   }

	return c;
}
nsl_matrix operator/(const nsl_matrix& a,const num_type b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator/\n";
        cout << "// (matrix) " << a.get_name() << " / (num_type) ";
        cout << b << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (b == 0)
	{
		cmd_error("*** nsl_matrix::operator/ ***");
		cmd_error("b: ",b);
		cmd_error("Division by zero");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] / b;

	return c;
}
nsl_matrix operator/(const num_type a,const  nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator/\n";
        cout << "// (num_type) " << a << " / (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
	   {
		if (b[i][j] == 0)
		{
			cmd_error("*** nsl_matrix::operator/ ***");
			cmd_error("Exit with the following status");
			b.print_status(cerr);
			b.print(cerr);
			cmd_error("Division by zero");
			return c;
		}
		c[i][j] = a / b[i][j];
	   }

	return c;
}
nsl_matrix operator/(const nsl_matrix& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator/\n";
        cout << "// (matrix) " << a.get_name() << " / (data) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	num_type   Vb = b.get_data();

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (b == 0)
	{
		cmd_error("*** nsl_matrix::operator/ ***");
		b.print_status(cerr);
		b.print(cerr);
		cmd_error("Division by zero");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] / b.get_data();

	return c;
}
nsl_matrix operator/(const nsl_data& a, const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator/\n";
        cout << "// (data) " << a.get_name() << " / (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
	   {
		if (b[i][j] == 0)
		{
			cmd_error("*** nsl_matrix::operator/ ***");
			b.print_status(cerr);
			b.print(cerr);
			cmd_error("Division by zero");
			return c;
		}
		c[i][j] = a.get_data() / b[i][j];
	   }

	return c;
}


// matrix pointwise division

nsl_matrix NSLdiv(const nsl_matrix& a,const nsl_matrix& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("*** nsl_matrix::NSLdiv ***");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		if (b[i][j] != 0)
			c[i][j] = a[i][j] / b[i][j];
		else
		{
			cmd_error("*** nsl_matrix::NSLdiv ***");
			b.print_status(cerr);
			b.print(cerr);			
			cmd_error("DIVISION by 0");
			return c;
		}

	return c;
}
nsl_matrix NSLdiv(const nsl_matrix& a,const num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (b == 0)
	{
		cmd_error("*** nsl_matrix::NSLdiv ***");	
		cmd_error("DIVISION by 0");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] / b;

	return c;
}
nsl_matrix NSLdiv(const num_type a, const nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		if (b[i][j] != 0)
			c[i][j] = a / b[i][j];
		else
		{
			cmd_error("*** nsl_matrix::NSLdiv ***");
			b.print_status(cerr);
			b.print(cerr);			
			cmd_error("DIVISION by 0");
			return c;
		}

	return c;
}
nsl_matrix NSLdiv(const nsl_matrix& a,const nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	if (b == 0)
	{
		cmd_error("*** nsl_matrix::NSLdiv ***");
		b.print_status(cerr);
		b.print(cerr);			
		cmd_error("DIVISION by 0");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] / b.get_data();

	return c;
}
nsl_matrix NSLdiv(const nsl_data& a, const nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		if (b[i][j] != 0)
			c[i][j] = a.get_data() / b[i][j];
		else
		{
			cmd_error("*** nsl_matrix::NSLdiv ***");
			b.print_status(cerr);
			b.print(cerr);			
			cmd_error("DIVISION by 0");
			return c;
		}

	return c;
}

*/