/* SCCS  @(#)nsl_vector_div_library.C	1.1---95/08/13--20:48:21 */
//
//	nsl_vector_div_library.C
//
/*
# include "nsl_all_kernel_include.h"

// vector pointwise division

nsl_vector operator/(const nsl_vector& a,const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator/\n";
	cout << "// (vector) " << a.get_name() << " / (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		if (b[i] != 0)
			c[i] = a[i] / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			a.print_status(cerr);
			b.print_status(cerr);
			return c;
		}			

	return c;
}
nsl_vector operator/(const nsl_vector& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator/\n";
	cout << "// (vector) " << a.get_name() << " / (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	if (b == 0)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		a.print_status(cerr);
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] / b;

	return c;
}
nsl_vector operator/(const num_type a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator/\n";
	cout << "// (num_type) " << a << " / (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		if (b[i] != 0)
			c[i] = a / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			b.print_status(cerr);
			return c;
		}			

	return c;
}
nsl_vector operator/(const nsl_vector& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator/\n";
	cout << "// (vector) " << a.get_name() << " / (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	if (b == 0)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		a.print_status(cerr);
		b.print_status(cerr);
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] / b.get_data();

	return c;
}
nsl_vector operator/(const nsl_data& a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator/\n";
	cout << "// (data) " << a.get_name() << " / (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		if (b[i] != 0)
			c[i] = a.get_data() / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			a.print_status(cerr);
			b.print_status(cerr);
			return c;
		}	

	return c;
}

// vector pointwise division

nsl_vector NSLdiv(const nsl_vector& a,const nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::NSLdiv ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		if (b[i] != 0)
			c[i] = a[i] / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			a.print_status(cerr);
			b.print_status(cerr);
			return c;
		}	

	return c;
}
nsl_vector NSLdiv(const nsl_vector& a,const num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	if (b == 0)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		a.print_status(cerr);
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] / b;

	return c;
}
nsl_vector NSLdiv(const num_type a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		if (b[i] != 0)
			c[i] = a / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			b.print_status(cerr);
			return c;
		}	

	return c;
}
nsl_vector NSLdiv(const nsl_vector& a,const nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	if (b == 0)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		a.print_status(cerr);
		b.print_status(cerr);
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] / b.get_data();

	return c;
}
nsl_vector NSLdiv(const nsl_data& a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		if (b[i] != 0)
			c[i] = a.get_data() / b[i];
		else
		{
			cmd_error("*** nsl_vector::operator/ ***");
			b.print_status(cerr);
			return c;
		}	

	return c;
}
*/
